package com.cmeeting.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cmeeting.dto.UserDTO;
import com.cmeeting.mapper.primary.UserIdMapper;
import com.cmeeting.pojo.MeetingInfo;
import com.cmeeting.service.*;
import com.cmeeting.vo.TencentMeetingVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Component
@Slf4j
public class CmeetingJob {
    @Autowired
    private WeComService weComService;
    @Autowired
    private TencentMeetingService tencentMeetingService;
    @Autowired
    private MeetingInfoService meetingInfoService;
    @Autowired
    private FileProcessProducer producer;
    @Autowired
    private MeetingRecordTemplateService meetingRecordTemplateService;

//    @PostConstruct
//    public void weComUserInit(){
//        weComUserSync();
//    }
//
//    @PostConstruct
//    public void tencentUserInit(){
//        TencentUserSync();
//    }
//    @PostConstruct
//    public void userBindInit(){
//        userBind();
//    }

    /**
     * 企微人员定时同步
     */
    @Scheduled(cron = "0 30 6 * * ?")
    public void weComUserSync() {
        try {
            log.info("-------企微人员定时同步任务开始-------");
            log.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            weComService.doUsers();
            log.info("-------企微人员定时同步任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 腾讯会议人员定时同步
     */
    @Scheduled(cron = "0 0 7 * * ?")
//    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void TencentUserSync() {
        try {
            log.info("-------腾讯会议人员定时同步任务开始-------");
            log.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            tencentMeetingService.doUsers();
            log.info("-------腾讯会议人员定时同步任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 绑定企微和腾会人员关系
     */
    @Scheduled(cron = "0 20 7 * * ?")
    public void userBind() {
        log.info("-------关联企微腾会人员定时任务开始-------");
        log.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));
        weComService.userBind();
        log.info("-------关联企微腾会人员定时任务结束--------");
    }

//    @Scheduled(fixedRate = 20 * 60 * 1000,initialDelay = 2 * 60 * 1000)
    @Scheduled(fixedRate = 20 * 60 * 1000)
    public void execute() {
        // 定义时间格式化器
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        // 获取当前时间
        LocalDateTime now = LocalDateTime.now();
        // 往之前推两天
        LocalDateTime beforeDay = now.minusDays(2);

        // 转换为 Unix 时间戳（秒）并转为字符串
        Long nowTimestamp = now.atZone(ZoneId.systemDefault()).toEpochSecond();
        Long beforeDayTimestamp = beforeDay.atZone(ZoneId.systemDefault()).toEpochSecond();
        //日志记录
        log.info("起始时间: " + beforeDay.format(formatter) + " | Unix 时间戳: " + beforeDayTimestamp);
        log.info("结束时间: " + now.format(formatter) + " | Unix 时间戳: " + nowTimestamp);
        log.info("----------------------------------");

        //智能体授权人员
        List<UserDTO> accessUserIds = tencentMeetingService.getAccessUserIds();
        if (CollectionUtils.isEmpty(accessUserIds)) {
            log.info("无生成纪要权限的人员");
            return;
        }else{
            log.info("生成纪要权限人员：->{}",accessUserIds.toString());
        }
        List<TencentMeetingVO.RecordFile> meetingFiles = tencentMeetingService.getMeetingFiles(accessUserIds);

        if (meetingFiles == null || meetingFiles.isEmpty()) {
            log.info("没有录制文件需要处理");
            return;
        }

        //获取模板授权的人员
        List<UserDTO.TemplateAuthorizedUserDTO> authorizedUsers = meetingRecordTemplateService.selectAuthorizedUsers();

        // 提交处理任务
        producer.submitBatchTasks(meetingFiles,authorizedUsers,Boolean.FALSE);
    }


    /**
     * 定时扫描早于一小时之前的，所有未重试过的会议，重新生成纪要
     */
//    @Scheduled(fixedRate = 30 * 60 * 1000,initialDelay = 10 * 60 * 1000)
//    @Scheduled(fixedRate = 30 * 60 * 1000)
    public void meetingMinutesRetry() {
        try {
            log.info("-------生成纪要重试定时任务开始-------");
            log.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            //查出所有早于一小时前的，生成失败且未重试过的会议
            List<MeetingInfo> meetingInfoList =
                    meetingInfoService.list(new LambdaQueryWrapper<MeetingInfo>()
                            .eq(MeetingInfo::getIsGenerated,Boolean.FALSE)
                            .eq(MeetingInfo::getGenerateRetry,Boolean.FALSE)
                            .le(MeetingInfo::getSyncTime,LocalDateTime.now().minusHours(1))
                    );

            if (meetingInfoList == null || meetingInfoList.isEmpty()) {
                log.info("没有生成失败的会议需要重试");
                return;
            }
            List<TencentMeetingVO.RecordFile> meetingFiles = new ArrayList<>();
            for (MeetingInfo meetingInfo : meetingInfoList) {
                TencentMeetingVO.RecordFile recordFile = TencentMeetingVO.RecordFile.builder()
                        .meetingId(meetingInfo.getMeetingId())
                        .subMeetingId(meetingInfo.getSubMeetingId())
                        .recordFileIdList(Arrays.asList(meetingInfo.getRecordFileId().split(","))).build();
                meetingFiles.add(recordFile);
            }

//            List<UserDTO> accessUserIds = tecentMeetingService.getAccessUserIds();

            //获取模板授权的人员
            List<UserDTO.TemplateAuthorizedUserDTO> authorizedUsers = meetingRecordTemplateService.selectAuthorizedUsers();

            // 提交处理任务
            producer.submitBatchTasks(meetingFiles,authorizedUsers, Boolean.TRUE);
            log.info("-------生成纪要重试定时任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 定时扫描早于一小时之前的，所有邮件推送未重试过的会议，重新推送邮件
     */
//    @Scheduled(fixedRate = 30 * 60 * 1000,initialDelay = 15 * 60 * 1000)
//    @Scheduled(fixedRate = 30 * 60 * 1000)
    public void emailPushRetry() {
        try {
            log.info("-------邮件推送重试定时任务开始-------");
            log.info("当前时间: " + LocalDate.now().format(DateTimeFormatter.ISO_LOCAL_DATE));

            //查出所有早于一小时前的，邮件推送失败且未重试过的会议
            List<MeetingInfo> meetingInfoList =
                    meetingInfoService.list(new LambdaQueryWrapper<MeetingInfo>()
                            .eq(MeetingInfo::getIsGenerated,Boolean.TRUE)
                            .eq(MeetingInfo::getEmailPushAccess,Boolean.TRUE)
                            .eq(MeetingInfo::getIsPushed,Boolean.FALSE)
                            .eq(MeetingInfo::getPushRetry,Boolean.FALSE)
                            .le(MeetingInfo::getSyncTime,LocalDateTime.now().minusHours(1))
                    );

            if (meetingInfoList == null || meetingInfoList.isEmpty()) {
                log.info("没有推送失败的邮件需要重试");
                return;
            }
            List<TencentMeetingVO.RecordFile> meetingFiles = new ArrayList<>();
            for (MeetingInfo meetingInfo : meetingInfoList) {
                TencentMeetingVO.RecordFile recordFile = TencentMeetingVO.RecordFile.builder()
                        .meetingId(meetingInfo.getMeetingId())
                        .subMeetingId(meetingInfo.getSubMeetingId()).build();
                meetingFiles.add(recordFile);
            }

            // 提交处理任务
            producer.submitEmailPushTasks(meetingFiles);
            log.info("-------邮件推送重试定时任务结束--------");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
