package com.cmeeting.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.dto.UserDTO;
import com.cmeeting.mapper.primary.CommonMapper;
import com.cmeeting.mapper.primary.MeetingRecordTemplateMapper;
import com.cmeeting.mapper.primary.RecordTemplatePermissionMapper;
import com.cmeeting.mapper.secondary.SysUserSysMapper;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.pojo.RecordTemplatePermission;
import com.cmeeting.service.MeetingRecordTemplateService;
import com.cmeeting.util.MinioUtils;
import com.cmeeting.vo.RecordTemplateVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class MeetingRecordTemplateServiceImpl extends ServiceImpl<MeetingRecordTemplateMapper, MeetingRecordTemplate>  implements MeetingRecordTemplateService {

    @Autowired
    private MinioUtils minioUtils;
    @Resource
    private RecordTemplatePermissionMapper permissionMapper;
    @Resource
    private SysUserSysMapper sysUserSysMapper;

    @Override
    public IPage<MeetingRecordTemplate> getPage(RecordTemplateVO vo) {
        LambdaQueryWrapper<MeetingRecordTemplate> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MeetingRecordTemplate::getIsDel,Boolean.FALSE);
        queryWrapper.eq(vo.getType() != null,MeetingRecordTemplate::getType,vo.getType());
        queryWrapper.like(vo.getName() != null,MeetingRecordTemplate::getName,vo.getName());
        queryWrapper.like(vo.getMeetingType() != null,MeetingRecordTemplate::getMeetingType,vo.getMeetingType());
        IPage<MeetingRecordTemplate> resultPage = baseMapper.selectPage(new Page<>(vo.getCurrent(), vo.getSize()), queryWrapper);
        return resultPage;
    }

    @Override
    public IPage<MeetingRecordTemplate> getPersonalPage(RecordTemplateVO vo) {
        LambdaQueryWrapper<MeetingRecordTemplate> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(MeetingRecordTemplate::getIsDel,Boolean.FALSE);
        queryWrapper.eq(vo.getType() != null,MeetingRecordTemplate::getType,vo.getType());
        queryWrapper.like(vo.getName() != null,MeetingRecordTemplate::getName,vo.getName());
        queryWrapper.like(vo.getMeetingType() != null,MeetingRecordTemplate::getMeetingType,vo.getMeetingType());
        IPage<MeetingRecordTemplate> resultPage = baseMapper.selectPage(new Page<>(vo.getCurrent(), vo.getSize()), queryWrapper);
        return resultPage;
    }

    /**
     * 上传附件模板到minio
     * @param file
     * @param id
     * @return
     */
    @Override
    public String templateUpload(MultipartFile file, Integer id) {
        MeetingRecordTemplate detail = getById(id);
        String separator = "/";
        String point = ".";
        // 获取文件名
        String originalFilename = file.getOriginalFilename();
        // 获取文件后缀
        String originalSuffix = FileUtil.getSuffix(originalFilename);
        // 生成文件名
        String nowDate = DateUtil.format(new Date(), "yyyy-MM-dd");
        String uuid = IdUtil.fastSimpleUUID();
        String originalFilePath = separator + nowDate + separator + uuid + point + originalSuffix;
        // 获取文件流
        try(InputStream is = file.getInputStream()){
            minioUtils.upload(originalFilePath,is);
        }catch (Exception e){
            throw new RuntimeException(e.getMessage());
        }
        detail.setTemplate(originalFilePath);
        updateById(detail);
        return originalFilePath;
    }

    /**
     * 获取模板授权的人员
     * @return
     */
    @Override
    public List<UserDTO.TemplateAuthorizedUserDTO> selectAuthorizedUsers() {
        List<UserDTO.TemplateAuthorizedUserDTO> result = new ArrayList<>();
        //查出所有系统模板(暂定)
        List<MeetingRecordTemplate> meetingRecordTemplates = baseMapper.selectList(new LambdaQueryWrapper<MeetingRecordTemplate>().eq(MeetingRecordTemplate::getType, "system"));
        //查出所有系统模板授权信息
        List<RecordTemplatePermission> permissions = permissionMapper.selectList(new LambdaQueryWrapper<RecordTemplatePermission>().eq(RecordTemplatePermission::getTemplateType, "system"));
        //所有授权模板的人员
        for (MeetingRecordTemplate template : meetingRecordTemplates) {
            UserDTO.TemplateAuthorizedUserDTO resultItem = new UserDTO.TemplateAuthorizedUserDTO();
            resultItem.setRecordTemplate(template);
            //该系统模板授权部门
            List<RecordTemplatePermission> deptPermissions = permissions.stream().filter(item -> "1".equals(item.getRelType()) && template.getId().equals(item.getTemplateId())).collect(Collectors.toList());
            //该系统模板授权人员
            List<RecordTemplatePermission> userPermissions = permissions.stream().filter(item -> "2".equals(item.getRelType()) && template.getId().equals(item.getTemplateId())).collect(Collectors.toList());

            List<String> authorizedUsers = new ArrayList<>();
            if(!CollectionUtils.isEmpty(userPermissions)){
                for (RecordTemplatePermission userPermission : userPermissions) {
                    authorizedUsers.add(userPermission.getRelId());
                }
            }
            if(!CollectionUtils.isEmpty(deptPermissions)){
                List<String> deptPath = new ArrayList<>();
                for (RecordTemplatePermission deptPermission : deptPermissions) {
                    String deptId = deptPermission.getRelId();
                    getDeptPath(deptPath,deptId);
                }
                List<String> usersByDept = sysUserSysMapper.getUsersByDept(deptPath);
                if(!CollectionUtils.isEmpty(usersByDept))
                    authorizedUsers.addAll(usersByDept);
            }
            resultItem.setUserIdList(authorizedUsers);
            result.add(resultItem);
        }
        return result;
    }

    /**
     * 获取部门的路径
     * @param deptPath
     * @param deptId
     */
    private void getDeptPath(List<String> deptPath, String deptId) {
        if(!deptPath.contains(deptId)) deptPath.add(deptId);
        List<String> subDeptIds = sysUserSysMapper.getSubDeptId(deptId);
        if(CollectionUtils.isEmpty(subDeptIds)) return;

        for (String subDeptId : subDeptIds) {
            //部门id去重
            if(!deptPath.contains(subDeptId)){
                deptPath.add(subDeptId);
                getDeptPath(deptPath,subDeptId);
            }
        }

    }
}