package com.cmeeting.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.mapper.primary.RecordTemplatePermissionMapper;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.pojo.RecordTemplatePermission;
import com.cmeeting.service.MeetingRecordTemplateService;
import com.cmeeting.service.RecordTemplatePermissionService;
import com.cmeeting.vo.RecordTemplateVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class RecordTemplatePermissionServiceImpl extends ServiceImpl<RecordTemplatePermissionMapper, RecordTemplatePermission>  implements RecordTemplatePermissionService {

    @Resource
    private MeetingRecordTemplateService recordTemplateService;

    /**
     * 授权
     * @param vo
     */
    @Override
    public boolean auth(RecordTemplateVO.TemplatePermissionVO vo) {
        List<RecordTemplateVO.PermissionItem> permissionItems = vo.getPermissionItems();
        if(CollectionUtils.isEmpty(permissionItems) || vo.getTemplateId() == null){
            return false;
        }
        MeetingRecordTemplate template = recordTemplateService.getById(vo.getTemplateId());
        if(template == null){
            throw new RuntimeException("模板不存在");
        }
        List<RecordTemplatePermission> permissions = new ArrayList<>();
//        List<RecordTemplateVO.PermissionItem> deptPermissions = permissionItems.stream().filter(item -> "1".equals(item.getRelType())).collect(Collectors.toList());
//        List<RecordTemplateVO.PermissionItem> userPermissions = permissionItems.stream().filter(item -> "2".equals(item.getRelType())).collect(Collectors.toList());
//        for (RecordTemplateVO.PermissionItem userPermission : userPermissions) {
//            RecordTemplatePermission permission = RecordTemplatePermission.builder()
//                    .templateId(vo.getTemplateId())
//                    .templateType(template.getType())
//                    .relType(userPermission.getRelType())
//                    .relId(userPermission.getRelId())
//                    .createTime(now)
//                    .build();
//            permissions.add(permission);
//        }
//        List<String> deptPath = new ArrayList<>();
//        for (RecordTemplateVO.PermissionItem deptPermission : deptPermissions) {
//            String deptId = deptPermission.getRelId();
//            getDeptPath(deptPath,deptId);
//        }
//        List<String> accessUserIds = !CollectionUtils.isEmpty(deptPath) ? commonMapper.getUsersByDept(deptPath) : new ArrayList<>();
        LocalDateTime now = LocalDateTime.now();
        for (RecordTemplateVO.PermissionItem permissionItem : permissionItems) {
            RecordTemplatePermission permission = RecordTemplatePermission.builder()
                    .templateId(vo.getTemplateId())
                    .templateType(template.getType())
                    .relType(permissionItem.getRelType())
                    .relId(permissionItem.getRelId())
                    .createTime(now)
                    .build();
            permissions.add(permission);
        }
        baseMapper.batchInsert(permissions);
        return true;
    }

    /**
     * 查询某个模板已经授权的人员或部门
     * @param vo
     * @return
     */
    @Override
    public List<RecordTemplatePermission> authList(RecordTemplateVO.TemplateAuthVO vo) {
        List<RecordTemplatePermission> permissions = baseMapper.selectList(
                new LambdaQueryWrapper<RecordTemplatePermission>()
                        .eq(RecordTemplatePermission::getTemplateId,vo.getTemplateId()));
        return permissions;
    }
}